﻿// RandomGlobules (c) Charles Petzold, 2009
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace RandomGlobules
{
    public partial class MainPage : UserControl
    {
        Random rand = new Random();
        ulong count = 0;
        WriteableBitmap bitmap;

        public MainPage()
        {
            InitializeComponent();
            CompositionTarget.Rendering += OnRendering;
        }

        void OnGridSizeChanged(object sender, SizeChangedEventArgs args)
        {
            if (args.NewSize.IsEmpty)
                return;

            bitmap = new WriteableBitmap((int)args.NewSize.Width, 
                                         (int)args.NewSize.Height);
            img.Source = bitmap;
            count = 0;
        }

        void OnRendering(object sender, EventArgs args)
        {
            if (bitmap == null)
                return;

            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;

            PathGeometry pathGeometry = GetRandomGlobule(width, height);

            byte[] bytes = new byte[4];
            rand.NextBytes(bytes);
            Color clr = Color.FromArgb(bytes[0], bytes[1],
                                       bytes[2], bytes[3]);

            Path path = new Path()
            {
                Fill = new SolidColorBrush(clr),
                Data = pathGeometry
            };

            path.Measure(new Size(width, height));
            path.Arrange(new Rect(new Point(), path.DesiredSize));

            bitmap.Render(path, null);
            bitmap.Invalidate();
            txt.Text = (++count).ToString();
        }
        PathGeometry GetRandomGlobule(int width, int height)
        {
            // Three control points
            Point pt1 = new Point(rand.Next(width), rand.Next(height));
            Point pt3 = new Point(rand.Next(width), rand.Next(height));
            Point pt5 = new Point(rand.Next(width), rand.Next(height));

            // Three end-points
            Point pt0 = RandomBetweenPoint(pt5, pt1);
            Point pt2 = RandomBetweenPoint(pt1, pt3);
            Point pt4 = RandomBetweenPoint(pt3, pt5);

            PathGeometry pathGeometry = new PathGeometry();

            PathFigure pathFigure = new PathFigure()
            {
                StartPoint = pt0,
                IsClosed = true
            };
            pathGeometry.Figures.Add(pathFigure);

            PolyQuadraticBezierSegment polyQuadBezierSegment = new PolyQuadraticBezierSegment();
            polyQuadBezierSegment.Points.Add(pt1);
            polyQuadBezierSegment.Points.Add(pt2);
            polyQuadBezierSegment.Points.Add(pt3);
            polyQuadBezierSegment.Points.Add(pt4);
            polyQuadBezierSegment.Points.Add(pt5);
            polyQuadBezierSegment.Points.Add(pt0);
            pathFigure.Segments.Add(polyQuadBezierSegment);

            return pathGeometry;
        }

        Point RandomBetweenPoint(Point pt1, Point pt2)
        {
            double t = rand.NextDouble();

            return new Point(pt1.X * (1 - t) + pt2.X * t,
                             pt1.Y * (1 - t) + pt2.Y * t);
        }
    }
}
